@@ -20,7 +20,7 @@
 /**
  * SECTION:gdata-entry
  * @short_description: GData entry object
- * @stability: Unstable
+ * @stability: Stable
  * @include: gdata/gdata-entry.h
  *
  * #GDataEntry represents a single object on the online service, such as a playlist, video or calendar event. It is a snapshot of the
@@ -598,6 +598,7 @@ parse_json (GDataParsable *parsable, JsonReader *reader, gpointer user_data, GEr
 
 	if (gdata_parser_string_from_json_member (reader, "title", P_DEFAULT | P_NO_DUPES, &(priv->title), &success, error) == TRUE ||
 	    gdata_parser_string_from_json_member (reader, "id", P_NON_EMPTY | P_NO_DUPES, &(priv->id), &success, error) == TRUE ||
+	    gdata_parser_string_from_json_member (reader, "description", P_NONE, &(priv->summary), &success, error) == TRUE ||
 	    gdata_parser_int64_time_from_json_member (reader, "updated", P_REQUIRED | P_NO_DUPES, &(priv->updated), &success, error) == TRUE ||
 	    gdata_parser_string_from_json_member (reader, "etag", P_NON_EMPTY | P_NO_DUPES, &(priv->etag), &success, error) == TRUE) {
 		return success;
@@ -849,6 +850,40 @@ _gdata_entry_set_updated (GDataEntry *self, gint64 updated)
 	self->priv->updated = updated;
 }
 
+/*
+ * _gdata_entry_set_published:
+ * @self: a #GDataEntry
+ * @updated: the new published value
+ *
+ * Sets the value of the #GDataEntry:published property to @published.
+ *
+ * Since: 0.17.0
+ */
+void
+_gdata_entry_set_published (GDataEntry *self, gint64 published)
+{
+	g_return_if_fail (GDATA_IS_ENTRY (self));
+	self->priv->published = published;
+}
+
+/*
+ * _gdata_entry_set_id:
+ * @self: a #GDataEntry
+ * @id: (nullable): the new ID
+ *
+ * Sets the value of the #GDataEntry:id property to @id.
+ *
+ * Since: 0.17.0
+ */
+void
+_gdata_entry_set_id (GDataEntry *self, const gchar *id)
+{
+	g_return_if_fail (GDATA_IS_ENTRY (self));
+
+	g_free (self->priv->id);
+	self->priv->id = g_strdup (id);
+}
+
 /**
  * gdata_entry_get_published:
  * @self: a #GDataEntry
