@@ -20,7 +20,7 @@
 /**
  * SECTION:gdata-feed
  * @short_description: GData feed object
- * @stability: Unstable
+ * @stability: Stable
  * @include: gdata/gdata-feed.h
  *
  * #GDataFeed is a list of entries (#GDataEntry) returned as the result of a query to a #GDataService, or given as the input to another
@@ -413,7 +413,6 @@ typedef struct {
 	GDataQueryProgressCallback progress_callback;
 	gpointer progress_user_data;
 	guint entry_i;
-	gboolean is_async;
 } ParseData;
 
 static gboolean
@@ -613,8 +612,10 @@ parse_json (GDataParsable *parsable, JsonReader *reader, gpointer user_data, GEr
 
 			/* Parse the node, passing it the reader cursor. */
 			entry = GDATA_ENTRY (_gdata_parsable_new_from_json_node (entry_type, reader, NULL, error));
-			if (entry == NULL)
+			if (entry == NULL) {
+				json_reader_end_element (reader);
 				return FALSE;
+			}
 
 			/* Calls the callbacks in the main thread */
 			if (data != NULL)
@@ -646,28 +647,45 @@ post_parse_json (GDataParsable *parsable, gpointer user_data, GError **error)
 	return TRUE;
 }
 
+/* Internal helper method to set these properties. */
+void
+_gdata_feed_set_page_info (GDataFeed *self, guint total_results,
+                           guint items_per_page)
+{
+	g_return_if_fail (GDATA_IS_FEED (self));
+
+	self->priv->total_results = total_results;
+	self->priv->items_per_page = items_per_page;
+}
+
 /*
  * _gdata_feed_new:
+ * @feed_type: the type of #GDataFeed subclass
  * @title: the feed's title
  * @id: the feed's ID
  * @updated: when the feed was last updated
  *
- * Creates a new #GDataFeed with the bare minimum of data to be valid.
+ * Creates a new #GDataFeed or subclass with the bare minimum of data to be
+ * valid.
  *
  * Return value: a new #GDataFeed
  *
- * Since: 0.6.0
+ * Since: 0.17.0
  */
 GDataFeed *
-_gdata_feed_new (const gchar *title, const gchar *id, gint64 updated)
+_gdata_feed_new (GType feed_type,
+                 const gchar *title,
+                 const gchar *id,
+                 gint64 updated)
 {
 	GDataFeed *feed;
 
+	g_return_val_if_fail (g_type_is_a (feed_type, GDATA_TYPE_FEED), NULL);
 	g_return_val_if_fail (title != NULL, NULL);
 	g_return_val_if_fail (id != NULL, NULL);
 	g_return_val_if_fail (updated >= 0, NULL);
 
-	feed = g_object_new (GDATA_TYPE_FEED, NULL);
+	feed = g_object_new (feed_type, NULL);
 	feed->priv->title = g_strdup (title);
 	feed->priv->id = g_strdup (id);
 	feed->priv->updated = updated;
@@ -677,7 +695,7 @@ _gdata_feed_new (const gchar *title, const gchar *id, gint64 updated)
 
 GDataFeed *
 _gdata_feed_new_from_xml (GType feed_type, const gchar *xml, gint length, GType entry_type,
-                          GDataQueryProgressCallback progress_callback, gpointer progress_user_data, gboolean is_async, GError **error)
+                          GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
 	ParseData *data;
 	GDataFeed *feed;
@@ -687,7 +705,7 @@ _gdata_feed_new_from_xml (GType feed_type, const gchar *xml, gint length, GType
 	g_return_val_if_fail (g_type_is_a (entry_type, GDATA_TYPE_ENTRY), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	data = _gdata_feed_parse_data_new (entry_type, progress_callback, progress_user_data, is_async);
+	data = _gdata_feed_parse_data_new (entry_type, progress_callback, progress_user_data);
 	feed = GDATA_FEED (_gdata_parsable_new_from_xml (feed_type, xml, length, data, error));
 	_gdata_feed_parse_data_free (data);
 
@@ -696,7 +714,7 @@ _gdata_feed_new_from_xml (GType feed_type, const gchar *xml, gint length, GType
 
 GDataFeed *
 _gdata_feed_new_from_json (GType feed_type, const gchar *json, gint length, GType entry_type,
-                          GDataQueryProgressCallback progress_callback, gpointer progress_user_data, gboolean is_async, GError **error)
+                          GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
 	ParseData *data;
 	GDataFeed *feed;
@@ -706,7 +724,7 @@ _gdata_feed_new_from_json (GType feed_type, const gchar *json, gint length, GTyp
 	g_return_val_if_fail (g_type_is_a (entry_type, GDATA_TYPE_ENTRY), NULL);
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-	data = _gdata_feed_parse_data_new (entry_type, progress_callback, progress_user_data, is_async);
+	data = _gdata_feed_parse_data_new (entry_type, progress_callback, progress_user_data);
 	feed = GDATA_FEED (_gdata_parsable_new_from_json (feed_type, json, length, data, error));
 	_gdata_feed_parse_data_free (data);
 
@@ -1049,7 +1067,7 @@ _gdata_feed_add_entry (GDataFeed *self, GDataEntry *entry)
 }
 
 gpointer
-_gdata_feed_parse_data_new (GType entry_type, GDataQueryProgressCallback progress_callback, gpointer progress_user_data, gboolean is_async)
+_gdata_feed_parse_data_new (GType entry_type, GDataQueryProgressCallback progress_callback, gpointer progress_user_data)
 {
 	ParseData *data;
 	data = g_slice_new (ParseData);
@@ -1057,7 +1075,6 @@ _gdata_feed_parse_data_new (GType entry_type, GDataQueryProgressCallback progres
 	data->progress_callback = progress_callback;
 	data->progress_user_data = progress_user_data;
 	data->entry_i = 0;
-	data->is_async = is_async;
 
 	return data;
 }
@@ -1072,9 +1089,15 @@ static gboolean
 progress_callback_idle (ProgressCallbackData *data)
 {
 	data->progress_callback (data->entry, data->entry_i, data->total_results, data->progress_user_data);
+
+	return G_SOURCE_REMOVE;
+}
+
+static void
+progress_callback_data_free (ProgressCallbackData *data)
+{
 	g_object_unref (data->entry);
 	g_slice_free (ProgressCallbackData, data);
-	return FALSE;
 }
 
 void
@@ -1093,14 +1116,12 @@ _gdata_feed_call_progress_callback (GDataFeed *self, gpointer user_data, GDataEn
 		progress_data->entry_i = data->entry_i;
 		progress_data->total_results = MIN (self->priv->items_per_page, self->priv->total_results);
 
-		if (data->is_async == TRUE) {
-			/* Send the callback; use G_PRIORITY_DEFAULT rather than G_PRIORITY_DEFAULT_IDLE
-			* to contend with the priorities used by the callback functions in GAsyncResult */
-			g_idle_add_full (G_PRIORITY_DEFAULT, (GSourceFunc) progress_callback_idle, progress_data, NULL);
-		} else {
-			/* If we're running synchronously, just call the callbacks directly */
-			progress_callback_idle (progress_data);
-		}
+		/* Send the callback; use G_PRIORITY_DEFAULT rather than G_PRIORITY_DEFAULT_IDLE
+		 * to contend with the priorities used by the callback functions in GAsyncResult */
+		g_main_context_invoke_full (NULL, G_PRIORITY_DEFAULT,
+		                            (GSourceFunc) progress_callback_idle,
+		                            progress_data,
+		                            (GDestroyNotify) progress_callback_data_free);
 	}
 	data->entry_i++;
 }
