@@ -20,7 +20,7 @@
 /**
  * SECTION:gdata-goa-authorizer
  * @short_description: GData GOA authorization interface
- * @stability: Unstable
+ * @stability: Stable
  * @include: gdata/gdata-goa-authorizer.h
  *
  * #GDataGoaAuthorizer provides an implementation of the #GDataAuthorizer interface for authentication and authorization using GNOME Online Accounts
@@ -72,6 +72,7 @@
 #include "services/contacts/gdata-contacts-service.h"
 #include "services/documents/gdata-documents-service.h"
 #include "services/picasaweb/gdata-picasaweb-service.h"
+#include "services/freebase/gdata-freebase-service.h"
 
 #define HMAC_SHA1_LEN 20 /* bytes, raw */
 
@@ -313,7 +314,7 @@ gdata_goa_authorizer_add_oauth2_authorization (GDataAuthorizer *authorizer, Soup
 		return;
 	}
 
-	authorization = g_string_new ("OAuth ");
+	authorization = g_string_new ("Bearer ");
 	g_string_append (authorization, priv->access_token);
 
 	/* Use replace here, not append, to make sure there's only one "Authorization" header. */
@@ -390,6 +391,8 @@ gdata_goa_authorizer_set_goa_object (GDataGoaAuthorizer *self, GoaObject *goa_ob
 	if (goa_object_peek_photos (goa_object) != NULL) {
 		add_authorization_domains (self, GDATA_TYPE_PICASAWEB_SERVICE);
 	}
+
+	add_authorization_domains (self, GDATA_TYPE_FREEBASE_SERVICE);
 }
 
 static void
